#pragma once
#include "communication/bridge/connection_lifecycle/i_connector.h"
#include "util/i_scheduled_executor.h"
#include "communication/bridge/i_bridge_client.h"

using namespace std::chrono_literals;

namespace spin
{
    class Connector : public IConnector
    {
    public:
        Connector(std::shared_ptr<IScheduledExecutor> scheduledExecutor, std::shared_ptr<IBridgeClient> bridgeClient);
        void startConnecting() const override;
        void stopConnecting() const override;
        void setOnSuccessCallback(std::function<void()> onSuccessCallback);

    private:
        static constexpr auto initialDelay = 0ms;
        static constexpr auto period = 2000ms;

        void tryToConnect() const;
        void connectAndNotifyListener() const;

        std::shared_ptr<IScheduledExecutor> scheduledExecutor_;
        std::shared_ptr<IBridgeClient> bridgeClient_;
        std::function<void()> onSuccessCallback_ = [&]() {std::cout << "Connector::onSuccessCallback not set." << std::endl;};
    };
} // namespace spin
